NBht.boot <-
function(x, nboot=499, tolerance = 1e-9,
                      type = c("singlecore", "multicores"), 
                      ncore = 4, 
                      plot.out = F) {
  
  ny <- nobs(x)
  mu <- x$fitted.values
  xx <- model.matrix(x)
  xx <- xx[,!colnames(xx) %in% c("(Intercept)")]
  mf <- formula(x)
  
  NBclass <- class(x)
  if( any(NBclass %in% c("negbin")) ) {
    x$theta <- 1 / x$theta  
  }
  th <- x$theta

  # score stat for original sample
  z0 <- score.exp(x, tolerance=tolerance)$zval
  
  # score stat for bootstrapped sample
  type <- match.arg(type)
  
  z0.boot <- switch(
    type,
                    
    singlecore = sapply(
      as.list(1:nboot), function(k) {
        print(k)
        set.seed(k)
        y <- rnbinom(n = ny, mu = mu, size = 1/th)
        f <- glm.nb2(y ~ xx)
        score.exp(f, tolerance=tolerance)$zval
      }),
    
    multicores = parallelsugar::mclapply(
      as.list(1:nboot), mc.cores = ncore, function(k) {  
        set.seed(k)
        y <- rnbinom(n = ny, mu = mu, size = 1/th)
        f <- glm.nb2(y ~ xx)
        score.exp(f, tolerance=tolerance)$zval
      })
  )
  z0.boot <- unlist(z0.boot)
  
  if(plot.out==TRUE) {
    hist(z0.boot)
    abline(v=z0, col=2)
  }

  list(
    zval.boot = z0.boot,
    pval.boot = sum(z0 <= z0.boot) / (1 + length(z0.boot)) 
  )
}
