glm.nb2 <-
function (formula, data, weights, subset, na.action, start = NULL, 
                     etastart, mustart, control = glm.control(...), method = "glm.fit", 
                     model = TRUE, x = FALSE, y = TRUE, contrasts = NULL, ..., 
                     init.theta, link = log, 
                     type = c("mle", "mle.exp", "mme"), 
                     tolerance = 1e-9,
                     theta.fix = FALSE, iteration.plot = FALSE)  {
  
  
  ## corrected lines ##
  type <- match.arg(type)
  
  loglik <- function(n, th, mu, y, w) {
    sum(w * (lgamma(1/th + y) - lgamma(1/th) - lgamma(y + 1) +
               1/th * log(1/th) + y * log(mu + (y == 0)) - (1/th + y) * log(1/th + mu)) )
  }
  ## corrected lines ##
  
  
  link <- substitute(link)
  fam0 <- if (missing(init.theta)) 
    do.call("poisson", list(link = link))
  else do.call("negative.binomial", list(theta = init.theta, link = link))
  mf <- Call <- match.call()
  m <- match(c("formula", "data", "subset", "weights", "na.action", "etastart", "mustart", "offset"), 
             names(mf), 0)
  mf <- mf[c(1, m)]
  mf$drop.unused.levels <- TRUE
  mf[[1L]] <- quote(stats::model.frame)
  mf <- eval.parent(mf)
  Terms <- attr(mf, "terms")
  if (method == "model.frame") 
    return(mf)
  Y <- model.response(mf, "numeric")
  X <- if (!is.empty.model(Terms)) 
    model.matrix(Terms, mf, contrasts)
  else matrix(, NROW(Y), 0)
  w <- model.weights(mf)
  if (!length(w)) 
    w <- rep(1, nrow(mf))
  else if (any(w < 0)) 
    stop("negative weights not allowed")
  offset <- model.offset(mf)
  mustart <- model.extract(mf, "mustart")
  etastart <- model.extract(mf, "etastart")
  n <- length(Y)
  if (!missing(method)) {
    if (!exists(method, mode = "function")) 
      stop(gettextf("unimplemented method: %s", sQuote(method)), 
           domain = NA)
    glm.fitter <- get(method)
  }
  else {
    method <- "glm.fit"
    glm.fitter <- stats::glm.fit
  }
  if (control$trace > 1) 
    message("Initial fit:")
  
  
  ## init fit (by Poisson glm)
  fit <- glm.fitter(x = X, y = Y, w = w, start = start, etastart = etastart, 
                    mustart = mustart, offset = offset, family = fam0, 
                    control = list(maxit=control$maxit, epsilon=control$epsilon, trace=control$trace>1), 
                    intercept = attr(Terms, "intercept") > 0)
  class(fit) <- c("glm", "lm")
  mu <- fit$fitted.values
  
  
  ## corrected lines ##
  if (theta.fix == FALSE) {
    th <- switch(type,
                 mle = as.vector(theta.ML(Y, mu, iteration.plot=iteration.plot)$t0),
                 mle.exp = as.vector(theta.ML(Y, mu, iteration.plot=iteration.plot)$t0),
                 mme = as.vector(theta.MM(Y, mu, iteration.plot=iteration.plot)$t0)
    )
  } else {
    th <- theta.fix  
  }
  ## corrected lines ##
  
    
  if (control$trace > 1) 
    message(gettextf("Initial value for 'theta': %f", signif(th)), domain = NA)

    
  ## corrected lines ##
  fam <- do.call("negative.binomial", list(theta = 1/th, link = link))
  ## corrected lines ##
  
  
  iter <- 0
  d1 <- sqrt(2 * max(1, fit$df.residual))
  d2 <- del <- 1
  g <- fam$linkfun
  Lm <- loglik(n, th, mu, Y, w)
  Lm0 <- Lm + 2 * d1
  
  
  while ((iter <- iter + 1) <= control$maxit && (abs(Lm0 - Lm)/d1 + abs(del)/d2) > control$epsilon) {
    
    eta <- g(mu)  # log(mu)
    fit <- glm.fitter(x = X, y = Y, w = w, etastart = eta, 
                      offset = offset, family = fam, 
                      control = list(maxit=control$maxit, 
                                     epsilon=control$epsilon, trace=control$trace>1), 
                      intercept = attr(Terms, "intercept") > 0)
    t0 <- th
    
    
    ## corrected lines ##
    if (theta.fix == FALSE) {
      th <- switch(type,
                   mle = as.vector(theta.ML(Y, mu, iteration.plot=iteration.plot)$t0),
                   mle.exp = as.vector(theta.ML(Y, mu, iteration.plot=iteration.plot)$t0),
                   mme = as.vector(theta.MM(Y, mu, iteration.plot=iteration.plot)$t0)
      )
    } else {
      th <- theta.fix  
    }
  
    fam <- do.call("negative.binomial", list(theta = 1/th, link = link))
    ## corrected lines ##
    
    
    mu  <- fit$fitted.values
    del <- t0 - th
    
    Lm0 <- Lm
    Lm  <- loglik(n, th, mu, Y, w)
    
    if (control$trace) {
      Ls <- loglik(n, th, Y, Y, w)
      Dev <- 2 * (Ls - Lm)
      message(sprintf("Theta(%d) = %f, 2(Ls - Lm) = %f", iter, signif(th), signif(Dev)), domain = NA)
    }
    
  }
  
  
  if (!is.null(attr(th, "warn"))) 
    fit$th.warn <- attr(th, "warn")
  if (iter > control$maxit) {
    warning("alternation limit reached")
    fit$th.warn <- gettext("alternation limit reached")
  }
  if (length(offset) && attr(Terms, "intercept")) {
    null.deviance <- if (length(Terms)) 
      glm.fitter(X[, "(Intercept)", drop = FALSE], Y, w, 
                 offset = offset, family = fam, 
                 control = list(maxit = control$maxit, epsilon = control$epsilon, trace = control$trace > 1), 
                 intercept = TRUE)$deviance
    else fit$deviance
    fit$null.deviance <- null.deviance
  }
  class(fit) <- c("negbin2", "glm", "lm")
  fit$terms <- Terms
  fit$formula <- as.vector(attr(Terms, "formula"))
  Call$init.theta <- signif(as.vector(th), 10)
  Call$link <- link
  fit$call <- Call
  if (model) 
    fit$model <- mf
  fit$na.action <- attr(mf, "na.action")
  if (x) 
    fit$x <- X
  if (!y) 
    fit$y <- NULL
  fit$theta <- as.vector(th)
  # fit$SE.theta <- attr(th, "SE")
  fit$twologlik <- as.vector(2 * Lm)
  fit$aic <- -fit$twologlik + 2 * fit$rank + 2
  fit$contrasts <- attr(X, "contrasts")
  fit$xlevels <- .getXlevels(Terms, mf)
  fit$method <- method
  fit$control <- control
  fit$offset <- offset
  
  
  ## corrected lines ## 
  fit$SE.theta <- switch(type,
                         mle = se.ml(fit),
                         mle.exp = se.ml.exp(fit, tolerance=tolerance),
                         mme = se.mm(fit))
  ## corrected lines ## 
  
  
  fit
}
