score.exp <-
function(fit.object, type = c("add", "mul"), tolerance) {
  y  <- fit.object$y
  mu <- fit.object$fitted.values
  th <- fit.object$theta
  x  <- model.matrix(fit.object)
  w  <- fit.object$weights
  
  type <- match.arg(type)
    
  calc.add <- function(y, mu, th, x, w) {  # Additive random effect
    S  <- sum( ( (y - mu)^2 - (mu + th*mu*y) ) / (1 + th*mu)^2 )
    
    I.11 <- sum( ( (1 + 2*mu + 3*th*mu)*mu ) / (1 + th*mu)^2 )
    I.12 <- matrix(apply((mu * x)/(1 + th*mu), 2, sum), nrow=1, ncol=ncol(x))
    I.21 <- t(I.12)
    I.13 <- I.31 <- sum( mu^2/(1+th*mu)^2 )
    
    I.22.i <- mapply(function(x.i, w.i) {
      crossprod(t(x.i)) * w.i
    },
    split(x, row(x)), 
    as.list(fit.object$weights), 
    SIMPLIFY = F)
    I.22 <- Reduce(f = "+", x = I.22.i)
    
    I.23 <- matrix(rep(0, ncol(x)), nrow=ncol(x), ncol=1)
    I.32 <- matrix(rep(0, ncol(x)), nrow=1, ncol=ncol(x))
    
    i.exp <- function(mu.i) {
      i.33 <- 1
      j <- 0
      dy <- c()
      while(i.33 >= tolerance) {
        i.33 <- (1/th + j)^(-2) * pnbinom(q=j, size=1/th, mu=mu.i, lower.tail=F)
        j <- j + 1
        dy <- c(dy, i.33)
      }
      sum(dy)
    }
    I.33 <- sum( (1/th^4) * ( sapply(as.list(mu), function(x) i.exp(x)) - th*mu/(mu + 1/th) ) )
    
    Imat <- rbind(cbind(I.11, I.12, I.13),
                  cbind(I.21, I.22, I.23),
                  cbind(I.31, I.32, I.33))
    colnames(Imat) <- rownames(Imat) <- c("v", names(coef(fit.object)), "th")
    
    V <- Imat[1,1] - Imat[1,-1,drop=F] %*% solve(Imat[-1,-1,drop=F]) %*% Imat[-1,1,drop=F]
    if(V < 0) V <- 1e-4
    
    z0 <- c(S/sqrt(V))
    p0 <- pnorm(q = z0, lower.tail = F)
    return( list(zval = z0, pval = p0) )
  }
  
  calc.mul <- function(y, mu, th, x, w) {  # Multiplicative random effect
    S  <- sum( ( (1 + th) * (y - mu)^2 - th*y^2 - y ) / (1 + th*mu)^2 )
    
    I.11 <- sum( ( 2*(1 + th)*mu^2 ) / (1 + th*mu)^2 )
    I.12 <- matrix(rep(0, ncol(x)), nrow=1, ncol=ncol(x))
    I.21 <- matrix(rep(0, ncol(x)), nrow=ncol(x), ncol=1)
    I.13 <- I.31 <- sum( mu^2/(1+th*mu)^2 )

    I.22.i <- mapply(function(x.i, w.i) {
      crossprod(t(x.i)) * w.i
    },
    split(x, row(x)), 
    as.list(fit.object$weights), 
    SIMPLIFY = F)
    I.22 <- Reduce(f = "+", x = I.22.i)
    
    I.23 <- matrix(rep(0, ncol(x)), nrow=ncol(x), ncol=1)
    I.32 <- matrix(rep(0, ncol(x)), nrow=1, ncol=ncol(x))
    
    i.exp <- function(mu.i) {
      i.33 <- 1
      j <- 0
      dy <- c()
      while(i.33 >= tolerance) {
        i.33 <- (1/th + j)^(-2) * pnbinom(q=j, size=1/th, mu=mu.i, lower.tail=F)
        j <- j + 1
        dy <- c(dy, i.33)
      }
      sum(dy)
    }
    I.33 <- sum( (1/th^4) * ( sapply(as.list(mu), function(x) i.exp(x)) - th*mu/(mu + 1/th) ) )
    
    Imat <- rbind(cbind(I.11, I.12, I.13),
                  cbind(I.21, I.22, I.23),
                  cbind(I.31, I.32, I.33))
    colnames(Imat) <- rownames(Imat) <- c("v", names(coef(fit.object)), "th")
    
    V <- Imat[1,1] - Imat[1,-1,drop=F] %*% solve(Imat[-1,-1,drop=F]) %*% Imat[-1,1,drop=F]
    if(V < 0) V <- 1e-4
    
    z0 <- c(S/sqrt(V))
    p0 <- pnorm(q = z0, lower.tail = F)
    return( list(zval = z0, pval = p0) )
  }

  switch(type,
         add = calc.add(y, mu, th, x, w),
         mul = calc.mul(y, mu, th, x, w)
  )
}
