score.obs <-
function(fit.object, type = c("add", "mul")) {
  y  <- fit.object$y
  mu <- fit.object$fitted.values
  th <- fit.object$theta
  x  <- model.matrix(fit.object)
  
  type <- match.arg(type)
  
  calc.add <- function(y, mu, th, x) {  # Additive random effect
    S <- sum( ( (y - mu)^2 - (mu + th*mu*y) ) / (1 + th*mu)^2 )
    
    d1 <- ( (y - mu)^2 - (mu + th*mu*y) ) / (1 + th*mu)^2
    d2 <- t( x*((y - mu) * mu)/(mu + th*mu^2) )
    d3 <- (digamma(1/th+y) - digamma(1/th) + log(1/th) + 1 - log(1/th+mu) - (y+1/th)/(mu+1/th)) * (-1/th^2)
    
    I.11 <- sum(d1^2)
    I.22 <- d2 %*% t(d2)
    I.33 <- sum(d3^2)
    I.12 <- matrix((apply(d1*t(d2), 2, sum)), nrow=1)
    I.21 <- t(I.12)
    I.13 <- sum(d1*d3)
    I.31 <- I.13
    I.32 <- matrix((apply(d3*t(d2), 2, sum)), nrow=1)   
    I.23 <- t(I.32)   
    
    Imat <- rbind(cbind(I.11, I.12, I.13),
                  cbind(I.21, I.22, I.23),
                  cbind(I.31, I.32, I.33))
    colnames(Imat) <- rownames(Imat) <- c("v", names(coef(fit.object)), "th")
    
    V <- Imat[1,1] - Imat[1,-1,drop=F] %*% solve(Imat[-1,-1,drop=F]) %*% Imat[-1,1,drop=F]
    if(V < 0) V <- 1e-4
    
    z0 <- c(S/sqrt(V))
    p0 <- pnorm(q = z0, lower.tail = F)
    return( list(zval = z0, pval = p0) )
  }
  
  calc.mul <- function(y, mu, th, x) {  # Multiplicative random effect
    S <- sum( ( (1 + th) * (y - mu)^2 - th*y^2 - y ) / (1 + th*mu)^2 )
    
    d1 <- ( (1 + th) * (y - mu)^2 - th*y^2 - y ) / (1 + th*mu)^2 
    d2 <- t( x*((y - mu) * mu)/(mu + th*mu^2) )
    d3 <- (digamma(1/th+y) - digamma(1/th) + log(1/th) + 1 - log(1/th+mu) - (y+1/th)/(mu+1/th)) * (-1/th^2)
    
    I.11 <- sum(d1^2)
    I.22 <- d2 %*% t(d2)
    I.33 <- sum(d3^2)
    I.12 <- matrix((apply(d1*t(d2), 2, sum)), nrow=1)
    I.21 <- t(I.12)
    I.13 <- sum(d1*d3)
    I.31 <- I.13
    I.32 <- matrix((apply(d3*t(d2), 2, sum)), nrow=1)   
    I.23 <- t(I.32) 
    
    Imat <- rbind(cbind(I.11, I.12, I.13),
                  cbind(I.21, I.22, I.23),
                  cbind(I.31, I.32, I.33))
    colnames(Imat) <- rownames(Imat) <- c("v", names(coef(fit.object)), "th")
    
    V <- Imat[1,1] - Imat[1,-1,drop=F] %*% solve(Imat[-1,-1,drop=F]) %*% Imat[-1,1,drop=F]
    if(V < 0) V <- 1e-4
    
    z0 <- c(S/sqrt(V))
    p0 <- pnorm(q = z0, lower.tail = F)
    return( list(zval = z0, pval = p0) )
  }
  
  switch(type,
         add = calc.add(y, mu, th, x),
         mul = calc.mul(y, mu, th, x)
  )
}
