theta.ML <-
function(y, mu, 
                     iteration.plot,
                     lwr=1e-6, upr=10000) {
  
  loglik <- function(th, mu, y) {
    sum( (lgamma(1/th + y) - lgamma(1/th) - lgamma(y + 1) +
            1/th * log(1/th) + y * log(mu + (y == 0)) - (1/th + y) * log(1/th + mu)) )
  }
  
  f <- function(x) {
    - loglik(x, mu, y)
  }
  
  t0 <- optimize(f, c(lwr, upr))$minimum
  
  if ( is.null(t0) || (t0 < 0) ) {  # check negative sign
    t0 <- 1/exp(6)
    warning("estimate truncated at zero")
    attr(t0, "warn") <- gettext("estimate truncated at zero")
  }
  
  # visualization for the estimation process
  xval <- seq(1e-6, 20, by = 0.1)
  yval <- sapply(as.list(xval), function(x) - f(x))
  
  if ( iteration.plot == TRUE ) {
    plot(x = xval, y = yval,
         xlab = expression(alpha), ylab = bquote(logLik(alpha)), 
         type = "l", lty = 2,
         ylim = c(min(range(yval)), max(range(yval))), 
         mgp=c(2.5, 1, 0) ) 
    
    mtext(bquote(paste(hat(alpha)==.(round(t0, 4)))), 
          line = 0.5, side = 3, cex = 1)
    
    abline(h = -f(t0), v = t0, col = 2, lwd = 1, lty = 2)
  }
  
  return( list(t0=t0, xval=xval, yval=yval) )
}
