theta.MM <-
function(y, mu, 
                     iteration.plot,
                     lwr=1e-6, upr=10000) {
  
  f <- function(y, mu, th, h) sum( (y - mu)^2/(mu +   th * mu^2) - 1 )
  
  t0 <- tryCatch(expr = {uniroot(f = f, y = y, mu = mu, interval = c(lwr, upr))$root},
                 error = function(e){ })  # searching uniroot
  
  if ( is.null(t0) || (t0 < 0) ) {  # check negative sign
    t0 <- 1/exp(6)
    warning("estimate truncated at zero")
    attr(t0, "warn") <- gettext("estimate truncated at zero")
  }
  
  # visualization for the estimation process
  xval <- seq(1e-6, 20, by = 0.1)
  yval <- sapply(as.list(xval), function(x) f(y, mu, x, h))
  
  if ( iteration.plot == TRUE ) {
    plot(x = xval, y = yval,
         xlab = expression(alpha), ylab = bquote(f(alpha)), 
         type = "l", lty = 2,
         ylim = c(min(c(0, range(yval))), max(c(0, range(yval)))), 
         mgp=c(2.5, 1, 0))
    
    mtext(bquote(paste(hat(alpha)==.(round(t0, 4)))), 
          line = 0.5, side = 3, cex = 1)
    
    abline(h = 0, v = t0, col = 2, lwd = 1, lty=2)
  }
  
  return( list(t0=t0, xval=xval, yval=yval) )
}
