\name{NBht}
\alias{NBht}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Calculate a score statistic to test the hidden heterogeneity in a fitted negative binomial regression model
}
\description{
Given a fitted negative binomial regression model, 
calculate a score test statistic and its p-value
}
\usage{
NBht(x, type.random = c("add", "mul"), 
        type.fisher = c("expected", "observed"),
        tolerance = 1e-9)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
A fitted negative binomial regression model assuming that
\eqn{Var(y) = \mu + \alpha \mu^2}.
}
  \item{type.random}{
type.random = "add" (default) assumes an additive random effect model.
type.random = "mul" assumes a multiplicative random effect model.
}
  \item{type.fisher}{
type.fisher = "expected" (default) uses an expected fisher information matrix for the calculation.
type.fisher = "observed" uses an observed fisher information matrix for the calculation.
}
  \item{tolerance}{
tolerance is the allowance error associated with the expected fisher information
for \eqn{\hat{\alpha}} calculated by Lawless (1987)'s method. 
tolerance controls the finite sums in the calculation. 
Smaller tolerance, more summations will be done. Default is 1e-9.
}
}
\details{
This function is applicable to the negative binomial regression model assuming that
\eqn{Var(y) = \mu + \alpha \mu^2}.
A negative binomial regression object assuming \eqn{Var(y) = \mu + \mu^2 / \kappa} 
is also applicable. NBht() can distinguish these two model object using different classes.
The former is considered as a \strong{negbin2} class, the latter is considered as a
a \strong{negbin} class.
}
\value{
zval : the value of the score test statistic.
pval : the p-value of the score test statistic.
}
\references{
Jeonghwan Kim and Woojoo Lee (2017), On testing the hidden heterogeneity in negative binomial regression models.
}
\author{
%%  ~~who you are~~
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
data(bioChemists)

library(MASS)
f0 <- glm.nb(art ~ fem + mar + kid5 + phd + ment,
              data = bioChemists)
NBht(f0)

f1 <- glm.nb2(art ~ fem + mar + kid5 + phd + ment,
              data = bioChemists)

NBht(f1)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ tests }
