\name{NBht.boot}
\alias{NBht.boot}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Parameteric bootstrap p-value of the score test statistic
}
\description{
Given the negative binomial regression model, 
calculate a p-value of the score test statistic by parameteric bootstrap method.
}
\usage{
NBht.boot(x, nboot = 499, tolerance = 1e-9,
          type = c("singlecore", "multicores"), 
          ncore = 4, 
          plot.out = F)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{
A fitted negative binomial regression model assuming that
\eqn{Var(y) = \mu + \alpha \mu^2}.
}
  \item{nboot}{
The number of bootstrapping.
}
  \item{tolerance}{
tolerance is the allowance error associated with the expected fisher information
for \eqn{\hat{\alpha}} calculated by Lawless (1987)'s method. 
tolerance controls the finite sums in the calculation. 
Smaller tolerance, more summations will be done. Default is 1e-9.
}
  \item{type}{
type = "singlecore" (default) uses a single core.
type = "multicores" uses multi-cores.
}
  \item{ncore}{
The number of cpu cores in parallel computation. Default is 4 (quad cores).
}
  \item{plot.out}{
plot.out = TRUE shows the position of the original score statistic
among bootstrapped statistics. Default is false.
}
}
\details{
This function is applicable to the negative binomial regression model assuming that
\eqn{Var(y) = \mu + \alpha \mu^2}.
A negative binomial regression object assuming \eqn{Var(y) = \mu + \mu^2 / \kappa} 
is also applicable. NBht.boot() can distinguish these two model object using different classes.
The former is considered as a \strong{negbin2} class, the latter is considered as a
a \strong{negbin} class.

For availability of parallel computations in window system,
this function need a mclapply() function in \strong{parallelsugar} package.
One needs do install \strong{parallelsugar} package manually before using the parallel function.
}
\value{
zval.boot : the parametric bootstrapped z-values.
pval.boot : the p-value by parametric bootstrap method.
}
\references{
Jeonghwan Kim and Woojoo Lee (2017), On testing the hidden heterogeneity in negative binomial regression models.
}
\author{
%%  ~~who you are~~
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
data(bioChemists)

f1 <- glm.nb2(art ~ fem + mar + kid5 + phd + ment,
              data = bioChemists)

# bootstrap p-value using single core
# time consuming works..
# NBht.boot(f1)  

# bootstrap p-value using multi-cores (parallel computation) 
# If necessary, do install parallelsugar package manually as follows,
# library(devtools)
# install_github('nathanvan/parallelsugar')

library(parallelsugar)
nc <- 4  # set cpu cores
NBht.boot(f1, nboot = 99, type = "multicores", ncore = nc)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ tests }
