\name{glm.nb2}
\alias{glm.nb2}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Fit a Negative Binomial Generalized Linear Model
}
\description{
A modification of the glm.nb() function in MASS package.
glm.nb() fits a negative binomial regression assuming that 
\deqn{Var(y) = \mu + \mu^2 / \kappa.}
In contrast to this, glm.nb2() function fits a negative binomial regression assuming that 
\deqn{Var(y) = \mu + \alpha \mu^2.}
}
\usage{
glm.nb2(formula, data, weights, subset, na.action, 
        start = NULL, etastart, mustart, 
        control = glm.control(...), method = "glm.fit", 
        model = TRUE, x = FALSE, y = TRUE, contrasts = NULL, ..., 
        init.theta, link = log, 
        type = c("mle", "mle.exp", "mme"), 
        tolerance = 1e-9,
        theta.fix = FALSE, iteration.plot = FALSE)
}
\arguments{
  \item{formula, data, weights, subset, ..., 
        init.theta, link = log}{
These are same arguments for the glm.nb() function. 
}
  \item{type}{
type = "mle" (default) estimates the dispersion parameter \eqn{\alpha}
by maximum likelihood estimation (MLE) and 
se(\eqn{\hat{\alpha}}) is calculated by using observed fisher information. 
type = "mle.exp" estimates the dispersion parameter \eqn{\alpha} 
in the same way when type = "mle" but 
se(\eqn{\hat{\alpha}}) is calculated by using expected fisher information.
type = "mme" estimates the dispersion parameter \eqn{\alpha}
by method of moment estimation (MME) and
se(\eqn{\hat{\alpha}}) is calculated by Lawless (1987)'s method.
}
  \item{tolerance}{
This option only works when type = "mle.exp".
tolerance is the allowance error associated with the expected fisher information
for \eqn{\hat{\alpha}} calculated by Lawless (1987)'s method. 
tolerance controls the finite sums in the calculation. 
Smaller tolerance, more summations will be done. Default is 1e-9.
}
  \item{theta.fix}{
When theta.fix = FALSE (default), the dispersion parameter \eqn{\alpha} 
will be estimated by mle or mme.
Otherwise, a negative binomial regression will be fitted with 
the user-defined fixed \eqn{\alpha}.
}
  \item{iteration.plot}{
iteration.plot = TRUE draws a likelihood plot for the dispersion parameter 
\eqn{\alpha} at each iteration in the estimation processes.
}
}
\details{
%%  ~~ If necessary, more details than the description above ~~
}
\value{
Values are almost same with those of glm.nb() except for the \strong{theta}.
The \strong{theta} in glm.nb() is \eqn{\kappa} of \eqn{Var(y) = \mu + \mu^2 / \kappa.}
In contrast to this, 
\strong{theta} in glm.nb2() is \eqn{\alpha} of \eqn{Var(y) = \mu + \alpha \mu^2.}
}
\references{
Venables, W. N. and Ripley, B. D. (2002) \emph{Modern Applied Statistics with S}. 
Fourth edition. Springer.  

Hilbe, J. M. (2011). \emph{Negative Binomial Regression}.
Second edition. Cambridge University Press.

Lawless, J. F. (1987). Negative binomial and mixed Poisson regression. 
The Canadian Journal of Statistics. 15(3), 209-225.
}
\author{
%%  ~~who you are~~
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
data(bioChemists)

f1 <- glm.nb2(art ~ fem + mar + kid5 + phd + ment,
              data = bioChemists)
summary(f1)

f1.mm <- glm.nb2(art ~ fem + mar + kid5 + phd + ment,
                 data = bioChemists,
                 type = "mme")
summary(f1.mm)


data(ausMTPL8486)

f2 <- glm.nb2(ClaimNb ~ log(AccNb) + offset(log(Pop)), 
              data = ausMTPL8486)
summary(f2)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ models }
